@@ -122,8 +122,10 @@ class CVEScanner:
 
             passed = False
             conditional = False
-
-            if version == "*":
+            #check version
+            if package_version.base_version == "*":
+                passed = True
+            elif version == "*":
                 # including
                 passed = version_range(package_version,
                                        version_start_including,
@@ -149,7 +151,7 @@ class CVEScanner:
                                                "version_range": v}
 
         # Go through and get all the severities
-        details = cve_db.get_cve_info_bulk(list(cve_report_list.keys()))
+        details = cve_db.get_cve_info_bulk(list(cve_report_list.keys()), self.score)
         cve_results = {}
         for k in cve_report_list.keys():
                 if k in details:
