@@ -22,7 +22,7 @@ from cve_ape.error_handler import (
     ErrorHandler,
     ErrorMode,
     NVDRateLimit,
-    SHAMismatch,
+    SHAMismatch, InsufficientArgs,
 )
 from cve_ape.log import LOGGER
 from cve_ape.nvd_api import NVD_API
@@ -39,6 +39,13 @@ DBNAME = "cve.db"
 OLD_CACHE_DIR = os.path.join(os.path.expanduser("~"), ".cache", "cvedb")
 NVD_FILENAME_TEMPLATE = "nvdcve-1.1-{}.json.gz"
 
+def db(f):
+    def wrapper(*args):
+        args[0].db_open()
+        d = f(*args)
+        args[0].db_close()
+        return d
+    return wrapper
 
 class Singleton(type):
     _instances = {}
@@ -373,22 +380,21 @@ class CVEDB(metaclass=Singleton):
                 schema_latest = True
         return schema_latest
 
+    @db
     def check_cve_entries(self):
         """Report if database has some CVE entries"""
-        self.db_open()
         cursor = self.connection.cursor()
         cve_entries_check = "SELECT COUNT(*) FROM cve_severity"
         cursor.execute(cve_entries_check)
         # Find number of entries
         cve_entries = cursor.fetchone()[0]
         self.LOGGER.info(f"There are {cve_entries} CVE entries in the database")
-        self.db_close()
         self.cve_count = cve_entries
         return cve_entries > 0
 
+    @db
     def init_database(self):
         """Initialize db tables used for storing cve/version data"""
-        self.db_open()
         cursor = self.connection.cursor()
         cve_data_create = """
         CREATE TABLE IF NOT EXISTS cve_severity (
@@ -429,6 +435,7 @@ class CVEDB(metaclass=Singleton):
             self.clear_cached_data()
         self.connection.commit()
 
+    @db
     def populate_db(self):
         """Function that populates the database from the JSON.
 
@@ -436,7 +443,6 @@ class CVEDB(metaclass=Singleton):
         in single nodes.  This isn't *required* by the json so may not be true everywhere.  If that's the case,
         we'll need a better parser to match those together.
         """
-        self.db_open()
         cursor = self.connection.cursor()
 
         insert_severity = """
@@ -496,8 +502,6 @@ class CVEDB(metaclass=Singleton):
         # supplemental data gets added here
         self.supplement_curl()
 
-        self.db_close()
-
     def populate_cve_list(
             self, all_cve_list, cursor, insert_severity, insert_cve_range, del_cve_range
     ):
@@ -614,11 +618,11 @@ class CVEDB(metaclass=Singleton):
                 affects_list.append(affects)
         return affects_list
 
+    @db
     def supplement_curl(self):
         """
         Get additional CVE data directly from the curl website amd add it to the cvedb
         """
-        self.db_open()
         insert_cve_range = """
         INSERT or REPLACE INTO cve_range(
             cve_number,
@@ -723,8 +727,8 @@ class CVEDB(metaclass=Singleton):
             self.LOGGER.warning(f"Deleting old cachedir {OLD_CACHE_DIR}")
             shutil.rmtree(OLD_CACHE_DIR)
 
-    def get_cve_info_bulk(self, cve_list):
-        self.db_open()
+    @db
+    def get_cve_info_bulk(self, cve_list, min_score: int = 0):
         cursor = self.connection.cursor()
         query_temptbl_create = """
         CREATE TEMP TABLE IF NOT EXISTS project_cve_list (
@@ -745,19 +749,17 @@ class CVEDB(metaclass=Singleton):
         query = """
         SELECT DISTINCT cve_severity.cve_number, severity, description, score, cvss_version, cvss_vector
                     FROM cve_severity INNER JOIN project_cve_list
-            WHERE cve_severity.cve_number = project_cve_list.cve_number
+            WHERE cve_severity.cve_number = project_cve_list.cve_number AND cve_severity.score >= ?
         """
-        cursor.execute(query)
+        cursor.execute(query, (min_score,))
         columnNames = [column[0] for column in cursor.description]
         cve_detailed_list = {}
         for row in cursor.fetchall():
             cve_detailed_list[row[0]] = dict(zip(columnNames, row))
-
-        self.db_close()
         return cve_detailed_list
 
+    @db
     def get_cves_for_packages(self, pkglist_package_names):
-        self.db_open()
         cursor = self.connection.cursor()
         query_temptbl_create = """
         CREATE TEMP TABLE IF NOT EXISTS project_cve_packages (
@@ -785,17 +787,15 @@ class CVEDB(metaclass=Singleton):
             project_cve_packages.version as pkglist_package_version,
             cve_range.vendor as vendor
             FROM project_cve_packages INNER JOIN cve_range
-            WHERE project_cve_packages.package = cve_range.product
+            WHERE project_cve_packages.package = cve_range.product 
         """
         cursor.execute(query)
         columnNames = [column[0] for column in cursor.description]
         cve_list = list(map(lambda x: dict(zip(columnNames, x)), cursor.fetchall()))
-
-        self.db_close()
         return cve_list
 
+    @db
     def get_packages_from_pkglist(self, pkglist_package_names):
-        self.db_open()
         cursor = self.connection.cursor()
         query_temptbl_create = """
         CREATE TEMP TABLE IF NOT EXISTS project_packages (
@@ -832,8 +832,6 @@ class CVEDB(metaclass=Singleton):
                 unknown_packages.append(package_name)
 
         self.LOGGER.warning("Unknown packages: {} out of {}".format(unknowns_counter, len(pkglist_package_names)))
-
-        self.db_close()
         return known_packages, unknown_packages
 
     def db_open(self):
@@ -872,3 +870,36 @@ class CVEDB(metaclass=Singleton):
 
     def __del__(self):
         self.rollback_cache_backup()
+
+    @db
+    def get_package(self, selected_package=None, selected_vendor=None):
+        cursor = self.connection.cursor()
+        if selected_vendor and selected_package:
+            query = """
+            SELECT DISTINCT cve_range.product, cve_range.vendor FROM cve_range
+            WHERE cve_range.product LIKE ? AND cve_range.vendor LIKE ?
+            """
+            cursor.execute(query, ("%{}%".format(selected_package), "%{}%".format(selected_vendor)))
+        elif selected_package:
+            query = """
+            SELECT DISTINCT cve_range.product, cve_range.vendor FROM cve_range
+            WHERE cve_range.product LIKE ?
+            """
+            cursor.execute(query, ("%{}%".format(selected_package), ))
+        elif selected_vendor:
+            query = """
+            SELECT DISTINCT cve_range.product, cve_range.vendor FROM cve_range
+            WHERE cve_range.vendor LIKE ?
+            """
+            cursor.execute(query, ("%{}%".format(selected_vendor), ))
+        else:
+            with ErrorHandler(logger=LOGGER, mode=ErrorMode.NoTrace):
+                raise InsufficientArgs(
+                    "The function cannot be called without vendor or package supplied"
+                )
+        known_packages = []
+        for x in cursor.fetchall():
+            data = {"name": x[0], "vendor": x[1], "version": "*"}
+            known_packages.append(data)
+        return known_packages
+
